import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { ArrowDropDown, ArrowDropUp, Error } from "@mui/icons-material";
import { IconButton, InputAdornment, TextField, Typography, } from "@mui/material";
import { AsYouType, formatIncompletePhoneNumber, getCountries, getCountryCallingCode, isValidPhoneNumber, ParseError, parsePhoneNumber, parsePhoneNumberWithError, validatePhoneNumberLength, } from "libphonenumber-js";
import { useEffect, useMemo, useState } from "react";
import { useTranslation } from "react-i18next";
import i18n from "../i18n";
import StyledAutocomplete from "./styledAutocomplete";
/**
 *
 * @param {string} countryCode Flag's corresponding country
 * @returns {string} Flag
 */
function getFlagEmoji(countryCode) {
    const codePoints = countryCode
        .toUpperCase()
        .split("")
        .map((char) => 127397 + char.charCodeAt(0));
    return String.fromCodePoint(...codePoints);
}
/**
 *
 * @param {NerdPhoneFieldProps} props Component props
 * @returns {ReactElement} NerdPhoneField
 */
export default function NerdPhoneField(props) {
    const { textFieldProps, onAccept, defaultCountry = "US", defaultValue, format = "international", } = props;
    const { t } = useTranslation("components", { i18n });
    const regionNames = useMemo(() => new Intl.DisplayNames([i18n.language], {
        type: "region",
    }), [i18n.language]);
    const countriesWithCallingCode = useMemo(() => {
        const data = [];
        const countries = getCountries();
        countries.forEach((country) => {
            data.push({
                countryCode: country,
                callingCode: getCountryCallingCode(country),
                fullName: regionNames.of(country) || "",
                flag: getFlagEmoji(country),
            });
        });
        return data;
    }, []);
    const [countryValue, setCountryValue] = useState(countriesWithCallingCode.find((obj) => {
        return (obj.countryCode ==
            (defaultValue != undefined && isValidPhoneNumber(defaultValue)
                ? parsePhoneNumber(defaultValue).country
                : defaultCountry));
    }));
    const [value, setValue] = useState("");
    const [errorMsg, setErrorMsg] = useState("");
    const [error, setError] = useState(false);
    const [anchorEl, setAnchorEl] = useState(null);
    const open = Boolean(anchorEl);
    /**
     * Close contries list
     */
    function handleClose() {
        setAnchorEl(null);
    }
    /**
     *
     * @param {ChangeEvent<HTMLInputElement | HTMLTextAreaElement>} e Event
     */
    function onTextFieldChange(e) {
        let newVal = e.target.value.replace(/[^+\d()[:space:]]/gm, "");
        try {
            const phoneNumber = parsePhoneNumberWithError(newVal, countryValue?.countryCode);
            //Update countryValue if inputted calling code is different
            if (phoneNumber.country) {
                if (phoneNumber.country != countryValue?.countryCode)
                    setCountryValue(countriesWithCallingCode.find((obj) => obj.countryCode == phoneNumber.country));
            }
            else {
                setCountryValue(undefined);
            }
            //Avoid formatting phone number when deleting parenthesis
            if (!(newVal.indexOf("(") != -1 && newVal.indexOf(")") == -1))
                newVal = formatIncompletePhoneNumber(newVal, phoneNumber.country);
        }
        catch (error) {
            if (!(error instanceof ParseError)) {
                throw error;
            }
        }
        setValue(newVal);
    }
    /**
     *
     * @param {CountryWithCallingCode} newValue New Value
     */
    function onAutocompleteChange(newValue) {
        if (countryValue && newValue.callingCode != countryValue.callingCode) {
            setValue("+" +
                newValue.callingCode +
                value.substring(countryValue.callingCode.length + 1, value.length));
        }
        setCountryValue(newValue);
    }
    useEffect(() => {
        if (defaultValue != undefined && defaultValue != value) {
            setValue(new AsYouType(defaultCountry).input(defaultValue || ""));
            if (isValidPhoneNumber(defaultValue)) {
                setCountryValue(countriesWithCallingCode.find((obj) => obj.countryCode == parsePhoneNumber(defaultValue).country));
            }
        }
    }, [defaultValue]);
    useEffect(() => {
        if (defaultCountry != undefined &&
            countryValue?.countryCode != defaultCountry) {
            const countryWithCallingCode = countriesWithCallingCode.find((obj) => obj.countryCode == defaultCountry);
            if (countryWithCallingCode) {
                setCountryValue(countryWithCallingCode);
                setValue("+" +
                    countryWithCallingCode.callingCode +
                    value.substring(countryWithCallingCode.callingCode.length + 1, value.length));
            }
        }
    }, [defaultCountry]);
    useEffect(() => {
        if (value != "") {
            const result = validatePhoneNumberLength(value, countryValue?.countryCode);
            let errorMsg = "";
            switch (result) {
                case "INVALID_COUNTRY":
                    errorMsg = t("nerdPhoneField.invalid_country");
                    break;
                case "INVALID_LENGTH":
                    errorMsg = t("nerdPhoneField.invalid_length");
                    break;
                case "NOT_A_NUMBER":
                    errorMsg = t("nerdPhoneField.invalid_phone_number");
                    break;
                case "TOO_LONG":
                    errorMsg = t("nerdPhoneField.too_long_phone_number");
                    break;
                case "TOO_SHORT":
                    errorMsg = t("nerdPhoneField.too_short_phone_number");
                    break;
                case undefined: {
                    const phoneNumber = parsePhoneNumber(value, countryValue?.countryCode);
                    onAccept?.(format == "international"
                        ? phoneNumber.number
                        : phoneNumber.nationalNumber);
                    break;
                }
                default:
                    break;
            }
            setErrorMsg(errorMsg);
            setError(!result && countryValue ? false : true);
        }
    }, [value, countryValue]);
    return (_jsx(TextField, { error: error, helperText: error ? errorMsg : "", ...textFieldProps, value: value, onChange: onTextFieldChange, InputProps: {
            startAdornment: (_jsxs(InputAdornment, { position: "start", children: [_jsxs("div", { style: {
                            display: "flex",
                            justifyContent: "center",
                            alignItems: "center",
                        }, children: [countryValue ? (_jsx(Typography, { color: "initial", children: countryValue.flag })) : (_jsx(Error, { color: "error", fontSize: "small" })), _jsx(IconButton, { onClick: (event) => setAnchorEl(event.currentTarget.parentElement), sx: { ml: 1 }, "aria-label": t("nerdPhoneField.select_country"), size: "small", children: open ? (_jsx(ArrowDropUp, { fontSize: "small" })) : (_jsx(ArrowDropDown, { fontSize: "small" })) })] }), _jsx(StyledAutocomplete, { anchorEl: anchorEl, handleClose: handleClose, onChange: (_e, newValue) => {
                            onAutocompleteChange(newValue);
                        }, open: open, options: countriesWithCallingCode, value: countryValue })] })),
        } }));
}
